# send_sms.py
# Sendet eine SMS-Nachricht an ein Handy
# Nur für Terminal-Betrieb, z. B. mit Thonny
# Keine deutschen Sonderzeichen (Äöß...)
# Mit Anzeige des Dialoges (print-Befehle können ggf. auskommentiert werden.)
# Getestet am 14.05.2025


#####################################################################################
#                            Import und Instanziierungen                            #
#####################################################################################

from time import sleep, ticks_ms, ticks_diff
from sys import exit

from machine import UART
uart1 = UART(1, baudrate = 56000, tx = 12, rx = 13)  # SIM800 hat autobauding von 1200 bis 57600
 
####################################################################################
#      Funktionen u.globale Variable zur Kommunikation mit dem SIM800L-Modul       #
####################################################################################

pin = b'9987' # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> aus-x-en
phone_number =b'01774738820' # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>aus-x-en
timeout = 40000 # in ms

def send_command(uart, cmd):       # cmd ist vom Typ Bytes, ohne \r 
    c = cmd + '\r'       # c ist Vom Typ Bytes
    anzahl_der_gesendeten_zeichen = uart.write(c)
    print(anzahl_der_gesendeten_zeichen, 'Zeichen gesendet:', c)
 
def read_response(uart, search_word, timeout): # Abbruch, wenn timeout ms überschritten
    received = b''
    t0 = ticks_ms()
    while not (search_word in received): 
        c = uart.read(1) # nur 1 Byte lesen
        if c != None:
            # print(c)
            received = received + c
            # print(received)
        t1 = ticks_ms()
        deltat = ticks_diff(t1, t0)
        if deltat > timeout:
            print('Timeout: Suchwort nicht in Anwort enthalten')
            print('Abbruch...')
            exit()
    return str(received, 'UTF-8')    
 
 
####################################################################################
#                              Hauptprogramm                                       #
####################################################################################

'''
Protokoll:
Kommando                // Erwartete Antwort
AT                      // OK\r\n
ATZ                     // OK\r\n
AT+CREG?                // +CREG: 0,1\r\n, wenn bereits eingeloggt, sonst +CREG: 0,0\r\n ... OK\r\n
AT+CPIN="xxxx"          // SMS Ready\r\n
AT+CMGF=1               // OK\r\n
AT+CSMP=17,167,0,0      // OK\r\n
AT+CMGS="0177xxxxx"     // >
Text + Ctrl-Z           // OK\r\n
'''

message = input('SMS-Nachricht:') # akzeptiert nur Standard-ASCII-Zeichen, also keine deutschen Sonderzeichen wie Ä, ü, ?...
message = bytes(message, 'UTF-8')

# Attention...
cmd = b'AT'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

# SIM800L resetten, ggf. zum Auslassen auskommentieren
cmd = b'ATZ'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

# eingeloggt?
cmd = b'AT+CREG?'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
logged = '0,1'
if logged in response:
    print('SIM bereits im Netz eingeloggt')
    print()
else: # einloggen...   
    cmd = b'AT+CPIN="'+pin+b'"'
    send_command(uart1, cmd)
    response = read_response(uart1, b'SMS Ready\r\n', timeout)
    print(response)
sleep(2)

# Message Format: Textmode...
cmd = b'AT+CMGF=1'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

# Text Mode: SMS-SUBMIT, Validity 1 day
cmd = b'AT+CSMP=17,167,0,0'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

# Adressat (Handy-Nr.)
cmd = b'AT+CMGS="'+phone_number+b'"'
send_command(uart1, cmd)
response = read_response(uart1, b'>', timeout)
print(response)
sleep(2)

# Nachricht
cmd = message + b'\x1A'  # \x1A = Ctrl-Z
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

print('Ende...')


