# sms_terminal.py
# Programm zum Anzeigen und Löschen von empfangenen Nachrichten (SMS)
# Nur für Terminal-Betrieb, z. B. mit Thonny
# getestet am 20.05.2025
 
 
#####################################################################################
#                            Import und Instanziierungen                            #
#####################################################################################

from time import sleep, ticks_ms, ticks_diff
from sys import exit

from machine import UART
uart1 = UART(1, baudrate = 56000, tx = 12, rx = 13)  # SIM800 hat autobauding von 1200 bis 57600
 
####################################################################################
#            Funktionen zur Kommunikation mit dem SIM800L-Modul                    #
####################################################################################

def send_command(uart, cmd):       # cmd ist vom Typ Bytes, ohne \r 
    c = cmd + '\r'       # c ist Vom Typ Bytes
    anzahl_der_gesendeten_zeichen = uart.write(c)
    print(anzahl_der_gesendeten_zeichen, 'Zeichen gesendet:', c)
 
def read_response(uart, suchwort, timeout): # Abbruch, wenn timeout ms überschritten
    received = b''
    t0 = ticks_ms()
    while not (suchwort in received) or (b'ERROR' in received):
        c = uart.read(1)
        if c != None:
            # print(c)
            received = received + c
            # print(received)
        t1 = ticks_ms()
        deltat = ticks_diff(t1, t0)
        if deltat > timeout:
            print('Timeout: Suchwort nicht erhalten')
            exit()
    return str(received, 'UTF-8')

 
####################################################################################
#                              Hauptprogramm                                       #
####################################################################################

'''
Einige Kommandos:
Kommando                // Erwartete Antwort
AT                      // OK\r\n
AT+CREG?                // +CREG: 0,1\r\n, wenn eingeloggt, sonst +CREG: 0,0\r\n ... OK\r\n
AT+CPIN="xxxx"          // SMS Ready\r\n
AT+CMGF=1               // OK\r\n
weitere Befehle und Antworten gemäß Eingabe (vgl. Kap. 5 von ...), z. B.
AT+CPMS=?               // +CPMS... OK
AT+CPMS="SM"            // +CPMS: 0,10,0,10,0,10\r\n OK\r\n (wenn SMS-Speicher leer), +CPMS: 1,10,1,10,1,10\r\n OK\r\n (wenn 1 SMS im SIM-Speicher)
AT+CMGR=x (0<x<=10)     // Nachricht Nr. x mit Absender, Datum und Uhrzeit, angeschlossen durch OK\r\n
AT+CMGD=x (0<x<=10)     // Nachricht Nr. x löschen, abgeschlossen durch OK\r\n 
'''

pin = b'9987' # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> aus-x-en
phone_number =b'01774738820' # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>aus-x-en
timeout = 20000 # in ms

cmd = b'AT'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

'''
cmd = b'ATZ' # Reset
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)
'''

# eingeloggt?
cmd = b'AT+CREG?'
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
logged = '0,1'
if logged in response:
    print('SIM bereits im Netz eingeloggt')
    print()
else: # einloggen...   
    cmd = b'AT+CPIN="9987"'
    send_command(uart1, cmd)
    response = read_response(uart1, b'SMS Ready\r\n', timeout)
    print(response)
sleep(2)

cmd = b'AT+CMGF=1' # text mode
send_command(uart1, cmd)
response = read_response(uart1, b'OK\r\n', timeout)
print(response)
sleep(2)

print('Terminal-Betrieb vorbereitet: eingeloggt und Text-Mode eingestellt...')
print()

weiter = True # anderen Variablennamen wählen...
while weiter:
    cmd = input('Kommando:')
    cmd = bytes(cmd, 'UTF-8') # String -> Bytes
    send_command(uart1, cmd)
    search_word = input('Suchwort:')
    search_word = bytes(search_word, 'UTF-8')
    search_word = search_word + b'\r\n'
    response = read_response(uart1, search_word, timeout)
    print(response)
    weiter = (input('weiter? j/n:') == 'j')
    print()
print('Ende...')


